#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/scrnsaver.h>
#include <Python.h>

int getSystemIdleTime(void) 
{
    time_t idle_time;
    static XScreenSaverInfo* mit_info;
    Display *display;
    int screen;

    mit_info = XScreenSaverAllocInfo();
    
    if((display = XOpenDisplay(NULL)) == NULL) 
    { 
        return(-1); 
    }

    screen = DefaultScreen(display);

    XScreenSaverQueryInfo(display, RootWindow(display,screen), mit_info);

    idle_time = (mit_info->idle);

    XFree(mit_info);
    XCloseDisplay(display);

    return idle_time;
}

static PyObject* get_idle_time(PyObject *self, PyObject *args)
{
    int value = getSystemIdleTime();
    return Py_BuildValue("i", value);
}

// Method definition object for this extension, these argumens mean:
// ml_name: The name of the method
// ml_meth: Function pointer to the method implementation
// ml_flags: Flags indicating special features of this method, such as
//          accepting arguments, accepting keyword arguments, being a
//          class method, or being a static method of a class.
// ml_doc:  Contents of this method's docstring
static PyMethodDef idletime_methods[] = 
{ 
    {   
        "get_idle_time", get_idle_time, METH_NOARGS, "Get the system idle time in milliseconds."
    },  
    {NULL, NULL, 0, NULL}
};

// Module definition
// The arguments of this structure tell Python what to call your extension,
// what it's methods are and where to look for it's method definitions
static struct PyModuleDef idletime_definition = 
{ 
    PyModuleDef_HEAD_INIT,
    "idletime",
    "A Python module that gives X11 idle time.",
    -1, 
    idletime_methods
};

// Module initialization
// Python calls this function when importing your extension. It is important
// that this function is named PyInit_[[your_module_name]] exactly, and matches
// the name keyword argument in setup.py's setup() call.
PyMODINIT_FUNC PyInit_idletime(void) 
{
    Py_Initialize();
    return PyModule_Create(&idletime_definition);
}